@@ -1,61 +1,68 @@
 from __future__ import annotations
 
 import contextlib
-import importlib
 import importlib.util
-import itertools
 import json
-import operator
 import os
 import site
 import sys
+import typing
+from collections.abc import Iterable
+from functools import cached_property
+from itertools import chain
 from pathlib import Path
 from sysconfig import get_paths, get_python_version, get_scheme_names
+from urllib.parse import urlparse
 
 import pipenv
 from pipenv.patched.pip._internal.commands.install import InstallCommand
 from pipenv.patched.pip._internal.index.package_finder import PackageFinder
-from pipenv.patched.pip._internal.req.req_uninstall import UninstallPathSet
-from pipenv.patched.pip._vendor import pkg_resources
+from pipenv.patched.pip._internal.req.req_install import InstallRequirement
+from pipenv.patched.pip._vendor.packaging.markers import UndefinedEnvironmentName
+from pipenv.patched.pip._vendor.packaging.specifiers import SpecifierSet
 from pipenv.patched.pip._vendor.packaging.utils import canonicalize_name
-from pipenv.utils.constants import is_type_checking
+from pipenv.patched.pip._vendor.packaging.version import Version
+from pipenv.patched.pip._vendor.packaging.version import parse as parse_version
+from pipenv.utils import console
+from pipenv.utils.fileutils import normalize_path, temp_path
+from pipenv.utils.funktools import chunked, unnest
 from pipenv.utils.indexes import prepare_pip_source_args
 from pipenv.utils.processes import subprocess_run
-from pipenv.utils.shell import make_posix, normalize_path
-from pipenv.vendor import click, vistir
+from pipenv.utils.shell import temp_environ
+from pipenv.utils.virtualenv import virtualenv_scripts_dir
+from pipenv.vendor.importlib_metadata.compat.py39 import normalized_name
+from pipenv.vendor.pipdeptree._models.dag import PackageDAG
+from pipenv.vendor.pipdeptree._models.package import InvalidRequirementError
+from pipenv.vendor.pythonfinder.utils import is_in_path
 
-try:
-    # this is only in Python3.8 and later
-    from functools import cached_property
-except ImportError:
-    # eventually distlib will remove cached property when they drop Python3.7
-    from pipenv.patched.pip._vendor.distlib.util import cached_property
+if sys.version_info < (3, 10):
+    from pipenv.vendor import importlib_metadata
+else:
+    import importlib.metadata as importlib_metadata
 
-
-if is_type_checking():
+if typing.TYPE_CHECKING:
     from types import ModuleType
-    from typing import ContextManager, Dict, Generator, List, Optional, Set, Union
-
-    import tomlkit
+    from typing import ContextManager, Generator
 
     from pipenv.project import Project, TPipfile, TSource
+    from pipenv.vendor import tomlkit
 
-BASE_WORKING_SET = pkg_resources.WorkingSet(sys.path)
+BASE_WORKING_SET = importlib_metadata.distributions()
 
 
 class Environment:
     def __init__(
         self,
-        prefix: Optional[str] = None,
-        python: Optional[str] = None,
+        prefix: str | None = None,
+        python: str | None = None,
         is_venv: bool = False,
-        base_working_set: pkg_resources.WorkingSet = None,
-        pipfile: Optional[Union[tomlkit.toml_document.TOMLDocument, TPipfile]] = None,
-        sources: Optional[List[TSource]] = None,
-        project: Optional[Project] = None,
+        base_working_set: list[importlib_metadata.Distribution] = None,
+        pipfile: tomlkit.toml_document.TOMLDocument | TPipfile | None = None,
+        sources: list[TSource] | None = None,
+        project: Project | None = None,
     ):
         super().__init__()
-        self._modules = {"pkg_resources": pkg_resources, "pipenv": pipenv}
+        self._modules = {"pipenv": pipenv}
         self.base_working_set = base_working_set if base_working_set else BASE_WORKING_SET
         prefix = normalize_path(prefix)
         self._python = None
@@ -72,8 +79,9 @@ class Environment:
             pipfile = project.parsed_pipfile
         self.pipfile = pipfile
         self.extra_dists = []
-        prefix = prefix if prefix else sys.prefix
-        self.prefix = Path(prefix)
+        if self.is_venv and prefix is not None and not Path(prefix).exists():
+            return
+        self.prefix = Path(prefix if prefix else sys.prefix)
         self._base_paths = {}
         if self.is_venv:
             self._base_paths = self.get_paths()
@@ -95,62 +103,21 @@ class Environment:
             module = importlib.import_module(name)
         return module
 
-    @classmethod
-    def resolve_dist(
-        cls, dist: pkg_resources.Distribution, working_set: pkg_resources.WorkingSet
-    ) -> Set[pkg_resources.Distribution]:
-        """Given a local distribution and a working set, returns all dependencies from the set.
-
-        :param dist: A single distribution to find the dependencies of
-        :type dist: :class:`pkg_resources.Distribution`
-        :param working_set: A working set to search for all packages
-        :type working_set: :class:`pkg_resources.WorkingSet`
-        :return: A set of distributions which the package depends on, including the package
-        :rtype: set(:class:`pkg_resources.Distribution`)
-        """
-
-        deps = set()
-        deps.add(dist)
-        try:
-            reqs = dist.requires()
-        # KeyError = limited metadata can be found
-        except (KeyError, AttributeError, OSError):  # The METADATA file can't be found
-            return deps
-        for req in reqs:
-            try:
-                dist = working_set.find(req)
-            except pkg_resources.VersionConflict:
-                # https://github.com/pypa/pipenv/issues/4549
-                # The requirement is already present with incompatible version.
-                continue
-            deps |= cls.resolve_dist(dist, working_set)
-        return deps
-
-    def extend_dists(self, dist: pkg_resources.Distribution) -> None:
-        extras = self.resolve_dist(dist, self.base_working_set)
-        self.extra_dists.append(dist)
-        if extras:
-            self.extra_dists.extend(extras)
-
-    def add_dist(self, dist_name: str) -> None:
-        dist = pkg_resources.get_distribution(pkg_resources.Requirement(dist_name))
-        self.extend_dists(dist)
-
     @cached_property
-    def python_version(self) -> str:
-        with self.activated():
-            sysconfig = self.safe_import("sysconfig")
-            py_version = sysconfig.get_python_version()
-            return py_version
-
-    def find_libdir(self) -> Optional[Path]:
-        libdir = self.prefix / "lib"
-        return next(iter(list(libdir.iterdir())), None)
+    def python_version(self) -> str | None:
+        with self.activated() as active:
+            if active:
+                # Extract version parts
+                version_str = f"{sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro}"
+                python_version = Version(version_str)  # Create PEP 440 compliant version
+                return str(python_version)  # Return the string representation
+            else:
+                return None
 
     @property
-    def python_info(self) -> Dict[str, str]:
+    def python_info(self) -> dict[str, str]:
         include_dir = self.prefix / "include"
-        if not os.path.exists(include_dir):
+        if not include_dir.exists():
             include_dirs = self.get_include_path()
             if include_dirs:
                 include_path = include_dirs.get(
@@ -167,15 +134,17 @@ class Environment:
         return {}
 
     def _replace_parent_version(self, path: str, replace_version: str) -> str:
-        if not os.path.exists(path):
-            base, leaf = os.path.split(path)
-            base, parent = os.path.split(base)
-            leaf = os.path.join(parent, leaf).replace(
+        path_obj = Path(path)
+        if not path_obj.exists():
+            parent = path_obj.parent
+            grandparent = parent.parent
+            leaf = f"{parent.name}/{path_obj.name}"
+            leaf = leaf.replace(
                 replace_version,
                 self.python_info.get("py_version_short", get_python_version()),
             )
-            return os.path.join(base, leaf)
-        return path
+            return str(grandparent / leaf)
+        return str(path_obj)
 
     @cached_property
     def install_scheme(self):
@@ -187,7 +156,7 @@ class Environment:
             return "posix_prefix"
 
     @cached_property
-    def base_paths(self) -> Dict[str, str]:
+    def base_paths(self) -> dict[str, str]:
         """
         Returns the context appropriate paths for the environment.
 
@@ -197,11 +166,6 @@ class Environment:
         .. note:: The implementation of this is borrowed from a combination of pip and
            virtualenv and is likely to change at some point in the future.
 
-        >>> from pipenv.core import project
-        >>> from pipenv.environment import Environment
-        >>> env = Environment(prefix=project.virtualenv_location, is_venv=True, sources=project.sources)
-        >>> import pprint
-        >>> pprint.pprint(env.base_paths)
         {'PATH': '/home/hawk/.virtualenvs/pipenv-MfOPs1lW/bin::/bin:/usr/bin',
         'PYTHONPATH': '/home/hawk/.virtualenvs/pipenv-MfOPs1lW/lib/python3.7/site-packages',
         'data': '/home/hawk/.virtualenvs/pipenv-MfOPs1lW',
@@ -216,7 +180,7 @@ class Environment:
         'stdlib': '/home/hawk/.pyenv/versions/3.7.1/lib/python3.7'}
         """
 
-        prefix = make_posix(self.prefix.as_posix())
+        prefix = Path(self.prefix)
         paths = {}
         if self._base_paths:
             paths = self._base_paths.copy()
@@ -256,24 +220,24 @@ class Environment:
         if not os.path.exists(paths["purelib"]) and not os.path.exists(paths["platlib"]):
             lib_paths = self.get_lib_paths()
             paths.update(lib_paths)
-        paths["PATH"] = paths["scripts"] + os.pathsep + os.defpath
+        paths["PATH"] = str(paths["scripts"]) + os.pathsep + os.defpath
         if "prefix" not in paths:
             paths["prefix"] = prefix
-        purelib = paths["purelib"] = make_posix(paths["purelib"])
-        platlib = paths["platlib"] = make_posix(paths["platlib"])
+        purelib = paths["purelib"] = Path(paths["purelib"])
+        platlib = paths["platlib"] = Path(paths["platlib"])
         if purelib == platlib:
-            lib_dirs = purelib
+            lib_dirs = [purelib]
         else:
-            lib_dirs = purelib + os.pathsep + platlib
+            lib_dirs = [purelib, platlib]
         paths["libdir"] = purelib
-        paths["PYTHONPATH"] = os.pathsep.join(["", ".", lib_dirs])
+        paths["PYTHONPATH"] = os.pathsep.join(["", ".", str(purelib), str(platlib)])
         paths["libdirs"] = lib_dirs
         return paths
 
     @cached_property
     def script_basedir(self) -> str:
         """Path to the environment scripts dir"""
-        prefix = make_posix(self.prefix.as_posix())
+        prefix = Path(self.prefix)
         paths = get_paths(
             self.install_scheme,
             vars={
@@ -286,19 +250,15 @@ class Environment:
     @property
     def python(self) -> str:
         """Path to the environment python"""
-        if self._python is not None:
-            return self._python
-        if os.name == "nt" and not self.is_venv:
-            py = Path(self.prefix).joinpath("python").absolute().as_posix()
-        else:
-            py = Path(self.script_basedir).joinpath("python").absolute().as_posix()
-        if not py:
-            py = Path(sys.executable).as_posix()
-        self._python = py
-        return py
+        if self._python is None:
+            self._python = (
+                (virtualenv_scripts_dir(self.prefix) / "python").absolute().as_posix()
+            )
+
+        return self._python
 
     @cached_property
-    def sys_path(self) -> List[str]:
+    def sys_path(self) -> list[str]:
         """
         The system path inside the environment
 
@@ -312,19 +272,12 @@ class Environment:
             return sys.path
         elif any([sys.prefix == self.prefix, not self.is_venv]):
             return sys.path
-        cmd_args = [self.python, "-c", "import json, sys; print(json.dumps(sys.path))"]
-        path, _ = vistir.misc.run(
-            cmd_args,
-            return_object=False,
-            nospin=True,
-            block=True,
-            combine_stderr=False,
-            write_to_stdout=False,
-        )
+
         try:
-            path = json.loads(path.strip())
-        except json.JSONDecodeError:
+            path = pipenv.utils.shell.load_path(self.python)
+        except json.decoder.JSONDecodeError:
             path = sys.path
+
         return path
 
     def build_command(
@@ -353,20 +306,21 @@ class Environment:
             "value = u'{0}'.format(json.dumps(paths)); print(value)"
         )
         sysconfig_line = "sysconfig.get_path('{0}')"
+
         if python_lib:
-            for key in ("purelib", "platlib", "stdlib", "platstdlib"):
-                pylib_lines.append(
-                    f"u'{key}': u'{{0}}'.format({sysconfig_line.format(key)})"
-                )
+            pylib_lines += [
+                f"u'{key}': u'{{0}}'.format({sysconfig_line.format(key)})"
+                for key in ("purelib", "platlib", "stdlib", "platstdlib")
+            ]
         if python_inc:
-            for key in ("include", "platinclude"):
-                pyinc_lines.append(
-                    f"u'{key}': u'{{0}}'.format({sysconfig_line.format(key)})"
-                )
+            pyinc_lines += [
+                f"u'{key}': u'{{0}}'.format({sysconfig_line.format(key)})"
+                for key in ("include", "platinclude")
+            ]
         lines = pylib_lines + pyinc_lines
         if scripts:
             lines.append(
-                "u'scripts': u'{0}'.format(%s)" % sysconfig_line.format("scripts")
+                "u'scripts': u'{{0}}'.format({})".format(sysconfig_line.format("scripts"))
             )
         if py_version:
             lines.append(
@@ -376,7 +330,7 @@ class Environment:
         py_command = py_command % lines_as_str
         return py_command
 
-    def get_paths(self) -> Optional[Dict[str, str]]:
+    def get_paths(self) -> dict[str, str] | None:
         """
         Get the paths for the environment by running a subcommand
 
@@ -391,7 +345,7 @@ class Environment:
         if c.returncode == 0:
             paths = json.loads(c.stdout)
             if "purelib" in paths:
-                paths["libdir"] = paths["purelib"] = make_posix(paths["purelib"])
+                paths["libdir"] = paths["purelib"] = Path(paths["purelib"])
             for key in (
                 "platlib",
                 "scripts",
@@ -401,14 +355,14 @@ class Environment:
                 "platinclude",
             ):
                 if key in paths:
-                    paths[key] = make_posix(paths[key])
+                    paths[key] = Path(paths[key])
             return paths
         else:
-            click.secho(f"Failed to load paths: {c.stderr}", fg="yellow")
-            click.secho(f"Output: {c.stdout}", fg="yellow")
+            console.print(f"Failed to load paths: {c.stderr}", style="yellow")
+            console.print(f"Output: {c.stdout}", style="yellow")
         return None
 
-    def get_lib_paths(self) -> Dict[str, str]:
+    def get_lib_paths(self) -> dict[str, str]:
         """Get the include path for the environment
 
         :return: The python include path for the environment
@@ -421,14 +375,14 @@ class Environment:
         if c.returncode == 0:
             paths = json.loads(c.stdout)
             if "purelib" in paths:
-                paths["libdir"] = paths["purelib"] = make_posix(paths["purelib"])
+                paths["libdir"] = paths["purelib"] = Path(paths["purelib"])
             for key in ("platlib", "platstdlib", "stdlib"):
                 if key in paths:
-                    paths[key] = make_posix(paths[key])
+                    paths[key] = Path(paths[key])
             return paths
         else:
-            click.secho(f"Failed to load paths: {c.stderr}", fg="yellow")
-            click.secho(f"Output: {c.stdout}", fg="yellow")
+            console.print(f"Failed to load paths: {c.stderr}", style="yellow")
+            console.print(f"Output: {c.stdout}", style="yellow")
         if not paths:
             if not self.prefix.joinpath("lib").exists():
                 return {}
@@ -459,7 +413,7 @@ class Environment:
                 return paths
         return {}
 
-    def get_include_path(self) -> Optional[Dict[str, str]]:
+    def get_include_path(self) -> dict[str, str] | None:
         """Get the include path for the environment
 
         :return: The python include path for the environment
@@ -472,11 +426,11 @@ class Environment:
             paths = json.loads(c.stdout)
             for key in ("include", "platinclude"):
                 if key in paths:
-                    paths[key] = make_posix(paths[key])
+                    paths[key] = Path(paths[key])
             return paths
         else:
-            click.secho(f"Failed to load paths: {c.stderr}", fg="yellow")
-            click.secho(f"Output: {c.stdout}", fg="yellow")
+            console.print(f"Failed to load paths: {c.stderr}", style="yellow")
+            console.print(f"Output: {c.stdout}", style="yellow")
         return None
 
     @cached_property
@@ -494,9 +448,9 @@ class Environment:
         return sys_prefix
 
     @cached_property
-    def paths(self) -> Dict[str, str]:
+    def paths(self) -> dict[str, str]:
         paths = {}
-        with vistir.contextmanagers.temp_environ(), vistir.contextmanagers.temp_path():
+        with temp_environ(), temp_path():
             os.environ["PYTHONIOENCODING"] = "utf-8"
             os.environ["PYTHONDONTWRITEBYTECODE"] = "1"
             paths = self.base_paths
@@ -525,41 +479,36 @@ class Environment:
         prefixes = [
             Path(prefix)
             for prefix in self.base_paths["libdirs"].split(os.pathsep)
-            if vistir.path.is_in_path(prefix, self.prefix.as_posix())
+            if is_in_path(prefix, self.prefix.as_posix())
         ]
         for loc in prefixes:
             if not loc.exists():
                 continue
             for pth in loc.iterdir():
-                if not pth.suffix == ".egg-link":
+                if pth.suffix != ".egg-link":
                     continue
                 contents = [
-                    vistir.path.normalize_path(line.strip())
-                    for line in pth.read_text().splitlines()
+                    normalize_path(line.strip()) for line in pth.read_text().splitlines()
                 ]
                 pth.write_text("\n".join(contents))
 
-    def get_distributions(self) -> Generator[pkg_resources.Distribution, None, None]:
+    def get_distributions(self) -> Generator[importlib_metadata.Distribution, None, None]:
         """
-        Retrives the distributions installed on the library path of the environment
+        Retrieves the distributions installed on the library path of the environment
 
         :return: A set of distributions found on the library path
         :rtype: iterator
         """
 
-        pip_target_dir = os.environ.get("PIP_TARGET")
-        libdirs = (
-            [pip_target_dir]
-            if pip_target_dir
-            else self.base_paths["libdirs"].split(os.pathsep)
-        )
-        dists = (pkg_resources.find_distributions(libdir) for libdir in libdirs)
-        yield from itertools.chain.from_iterable(dists)
+        libdirs = self.base_paths["libdirs"]
+        for libdir in libdirs:
+            dists = importlib_metadata.distributions(path=[str(libdir)])
+            yield from dists
 
-    def find_egg(self, egg_dist: pkg_resources.Distribution) -> str:
+    def find_egg(self, egg_dist: importlib_metadata.Distribution) -> str:
         """Find an egg by name in the given environment"""
         site_packages = self.libdir[1]
-        search_filename = f"{egg_dist.project_name}.egg-link"
+        search_filename = f"{normalized_name(egg_dist)}.egg-link"
         try:
             user_site = site.getusersitepackages()
         except AttributeError:
@@ -570,36 +519,36 @@ class Environment:
             if os.path.isfile(egg):
                 return egg
 
-    def locate_dist(self, dist: pkg_resources.Distribution) -> str:
+    def locate_dist(self, dist: importlib_metadata.Distribution) -> str:
         """Given a distribution, try to find a corresponding egg link first.
 
         If the egg - link doesn 't exist, return the supplied distribution."""
 
         location = self.find_egg(dist)
-        return location or dist.location
+        return location or dist._path
 
-    def dist_is_in_project(self, dist: pkg_resources.Distribution) -> bool:
+    def dist_is_in_project(self, dist: importlib_metadata.Distribution) -> bool:
         """Determine whether the supplied distribution is in the environment."""
-        from .environments import normalize_pipfile_path as _normalized
+        libdirs = self.base_paths["libdirs"]
+        location = Path(self.locate_dist(dist))
 
-        prefixes = [
-            _normalized(prefix)
-            for prefix in self.base_paths["libdirs"].split(os.pathsep)
-            if _normalized(prefix).startswith(_normalized(self.prefix.as_posix()))
-        ]
-        location = self.locate_dist(dist)
         if not location:
             return False
-        location = _normalized(make_posix(location))
-        return any(location.startswith(prefix) for prefix in prefixes)
 
-    def get_installed_packages(self) -> List[pkg_resources.Distribution]:
+        # Since is_relative_to is not available in Python 3.8, we use a workaround
+        if sys.version_info < (3, 9):
+            location_str = str(location)
+            return any(location_str.startswith(str(libdir)) for libdir in libdirs)
+        else:
+            return any(location.is_relative_to(libdir) for libdir in libdirs)
+
+    def get_installed_packages(self) -> list[importlib_metadata.Distribution]:
         """Returns all of the installed packages in a given environment"""
         workingset = self.get_working_set()
         packages = [
             pkg
             for pkg in workingset
-            if self.dist_is_in_project(pkg) and pkg.key != "python"
+            if self.dist_is_in_project(pkg) and normalized_name(pkg) != "python"
         ]
         return packages
 
@@ -622,13 +571,13 @@ class Environment:
 
     def get_package_info(
         self, pre: bool = False
-    ) -> Generator[pkg_resources.Distribution, None, None]:
+    ) -> Generator[importlib_metadata.Distribution, None, None]:
         packages = self.get_installed_packages()
 
         with self.get_finder() as finder:
             for dist in packages:
-                typ = "unknown"
-                all_candidates = finder.find_all_candidates(dist.key)
+                name = normalized_name(dist)
+                all_candidates = finder.find_all_candidates(name)
                 if not self.pipfile.get("pre", finder.allow_all_prereleases):
                     # Remove prereleases
                     all_candidates = [
@@ -639,29 +588,28 @@ class Environment:
 
                 if not all_candidates:
                     continue
-                candidate_evaluator = finder.make_candidate_evaluator(
-                    project_name=dist.key
-                )
+                candidate_evaluator = finder.make_candidate_evaluator(project_name=name)
                 best_candidate_result = candidate_evaluator.compute_best_candidate(
                     all_candidates
                 )
-                remote_version = best_candidate_result.best_candidate.version
+                remote_version = parse_version(
+                    str(best_candidate_result.best_candidate.version)
+                )
                 if best_candidate_result.best_candidate.link.is_wheel:
-                    typ = "wheel"
+                    pass
                 else:
-                    typ = "sdist"
+                    pass
                 # This is dirty but makes the rest of the code much cleaner
                 dist.latest_version = remote_version
-                dist.latest_filetype = typ
                 yield dist
 
     def get_outdated_packages(
         self, pre: bool = False
-    ) -> List[pkg_resources.Distribution]:
+    ) -> list[importlib_metadata.Distribution]:
         return [
             pkg
             for pkg in self.get_package_info(pre=pre)
-            if pkg.latest_version._key > pkg.parsed_version._key
+            if pkg.latest_version > parse_version(pkg.version)
         ]
 
     @classmethod
@@ -688,23 +636,32 @@ class Environment:
         return d
 
     def get_package_requirements(self, pkg=None):
-        from itertools import chain
-
-        from pipenv.vendor.pipdeptree import PackageDAG
-
         flatten = chain.from_iterable
 
         packages = self.get_installed_packages()
         if pkg:
-            packages = [p for p in packages if p.key == pkg]
+            packages = [p for p in packages if normalized_name(p) == pkg]
 
-        tree = PackageDAG.from_pkgs(packages).sort()
-        branch_keys = {r.key for r in flatten(tree.values())}
-        if pkg is not None:
-            nodes = [p for p in tree.keys() if p.key == pkg]
+        try:
+            tree = PackageDAG.from_pkgs(packages)
+        except InvalidRequirementError as e:
+            console.print(f"Invalid requirement: {e}", style="yellow")
+            tree = PackageDAG({})
+        except UndefinedEnvironmentName:
+            # Handle the case when 'extra' environment variable is not defined
+            tree = PackageDAG({})
+        except Exception as e:
+            # Handle any other exceptions that may occur during PackageDAG initialization
+            console.print(f"Failed to create PackageDAG: {e}", style="yellow")
+            tree = PackageDAG({})
+
+        tree = tree.sort()
+        branch_keys = {r.project_name for r in flatten(tree.values())}
+        if pkg is None:
+            nodes = [p for p in tree if p.project_name not in branch_keys]
         else:
-            nodes = [p for p in tree.keys() if p.key not in branch_keys]
-        key_tree = {k.key: v for k, v in tree.items()}
+            nodes = [p for p in tree if p.project_name == pkg]
+        key_tree = {k.project_name: v for k, v in tree.items()}
 
         return [self._get_requirements_for_package(p, key_tree) for p in nodes]
 
@@ -723,8 +680,6 @@ class Environment:
         yield new_node
 
     def reverse_dependencies(self):
-        from vistir.misc import chunked, unnest
-
         rdeps = {}
         for req in self.get_package_requirements():
             for d in self.reverse_dependency(req):
@@ -752,15 +707,11 @@ class Environment:
                 }
         return rdeps
 
-    def get_working_set(self):
-        """Retrieve the working set of installed packages for the environment.
-
-        :return: The working set for the environment
-        :rtype: :class:`pkg_resources.WorkingSet`
-        """
-
-        working_set = pkg_resources.WorkingSet(self.sys_path)
-        return working_set
+    def get_working_set(self) -> Iterable:
+        """Retrieve the working set of installed packages for the environment."""
+        if not hasattr(self, "sys_path"):
+            return []
+        return importlib_metadata.distributions(path=self.sys_path)
 
     def is_installed(self, pkgname):
         """Given a package name, returns whether it is installed in the environment
@@ -770,87 +721,41 @@ class Environment:
         :rtype: bool
         """
 
-        return any(d for d in self.get_distributions() if d.project_name == pkgname)
+        return any(d for d in self.get_distributions() if normalized_name(d) == pkgname)
 
-    def is_satisfied(self, req):
+    def is_satisfied(self, req: InstallRequirement):
         match = next(
             iter(
                 d
                 for d in self.get_distributions()
-                if canonicalize_name(d.project_name) == req.normalized_name
+                if req.name
+                and canonicalize_name(normalized_name(d)) == canonicalize_name(req.name)
             ),
             None,
         )
         if match is not None:
-            if req.editable and req.line_instance.is_local and self.find_egg(match):
-                requested_path = req.line_instance.path
-                return requested_path and os.path.samefile(requested_path, match.location)
-            elif match.has_metadata("direct_url.json"):
-                direct_url_metadata = json.loads(match.get_metadata("direct_url.json"))
-                commit_id = direct_url_metadata.get("vcs_info", {}).get("commit_id", "")
-                vcs_type = direct_url_metadata.get("vcs_info", {}).get("vcs", "")
-                _, pipfile_part = req.as_pipfile().popitem()
-                return (
-                    vcs_type == req.vcs
-                    and commit_id == req.commit_hash
-                    and direct_url_metadata["url"] == pipfile_part[req.vcs]
-                )
-            elif req.is_vcs or req.is_file_or_url:
-                return False
-            elif req.line_instance.specifiers is not None:
-                return req.line_instance.specifiers.contains(
+            if req.specifier is not None:
+                return SpecifierSet(str(req.specifier)).contains(
                     match.version, prereleases=True
                 )
+            if req.link is None:
+                return True
+            elif req.editable and req.link.is_file:
+                requested_path = req.link.file_path
+                if os.path.exists(requested_path):
+                    local_path = requested_path
+                else:
+                    parsed_url = urlparse(requested_path)
+                    local_path = parsed_url.path
+                return requested_path and os.path.samefile(local_path, match.location)
+            elif match.has_metadata("direct_url.json") or (req.link and req.link.is_vcs):
+                # Direct URL installs and VCS installs we assume are not satisfied
+                # since due to skip-lock we may be installing from Pipfile we have insufficient
+                # information to determine if a branch or ref has actually changed.
+                return False
             return True
         return False
 
-    def run(self, cmd, cwd=os.curdir):
-        """Run a command with :class:`~subprocess.Popen` in the context of the environment
-
-        :param cmd: A command to run in the environment
-        :type cmd: str or list
-        :param str cwd: The working directory in which to execute the command, defaults to :data:`os.curdir`
-        :return: A finished command object
-        :rtype: :class:`~subprocess.Popen`
-        """
-
-        c = None
-        with self.activated():
-            script = vistir.cmdparse.Script.parse(cmd)
-            c = vistir.misc.run(
-                script._parts,
-                return_object=True,
-                nospin=True,
-                cwd=cwd,
-                write_to_stdout=False,
-            )
-        return c
-
-    def run_py(self, cmd, cwd=os.curdir):
-        """Run a python command in the environment context.
-
-        :param cmd: A command to run in the environment - runs with `python -c`
-        :type cmd: str or list
-        :param str cwd: The working directory in which to execute the command, defaults to :data:`os.curdir`
-        :return: A finished command object
-        :rtype: :class:`~subprocess.Popen`
-        """
-
-        c = None
-        if isinstance(cmd, str):
-            script = vistir.cmdparse.Script.parse(f"{self.python} -c {cmd}")
-        else:
-            script = vistir.cmdparse.Script.parse([self.python, "-c"] + list(cmd))
-        with self.activated():
-            c = vistir.misc.run(
-                script._parts,
-                return_object=True,
-                nospin=True,
-                cwd=cwd,
-                write_to_stdout=False,
-            )
-        return c
-
     def run_activate_this(self):
         """Runs the environment's inline activation script"""
         if self.is_venv:
@@ -859,7 +764,7 @@ class Environment:
                 raise OSError(f"No such file: {activate_this!s}")
             with open(activate_this) as f:
                 code = compile(f.read(), activate_this, "exec")
-                exec(code, dict(__file__=activate_this))
+                exec(code, {"__file__": activate_this})
 
     @contextlib.contextmanager
     def activated(self):
@@ -881,10 +786,20 @@ class Environment:
         to `os.environ["PATH"]` to ensure that calls to `~Environment.run()` use the
         environment's path preferentially.
         """
+
+        # Fail if the virtualenv is needed but cannot be found
+        if self.is_venv and (
+            hasattr(self, "prefix")
+            and not self.prefix.exists()
+            or not hasattr(self, "prefix")
+        ):
+            yield False
+            return
+
         original_path = sys.path
         original_prefix = sys.prefix
         prefix = self.prefix.as_posix()
-        with vistir.contextmanagers.temp_environ(), vistir.contextmanagers.temp_path():
+        with temp_environ(), temp_path():
             os.environ["PATH"] = os.pathsep.join(
                 [
                     self.script_basedir,
@@ -897,116 +812,15 @@ class Environment:
             if self.is_venv:
                 os.environ["PYTHONPATH"] = self.base_paths["PYTHONPATH"]
                 os.environ["VIRTUAL_ENV"] = prefix
-            else:
-                if not self.project.s.PIPENV_USE_SYSTEM and not os.environ.get(
-                    "VIRTUAL_ENV"
-                ):
-                    os.environ["PYTHONPATH"] = self.base_paths["PYTHONPATH"]
-                    os.environ.pop("PYTHONHOME", None)
+            elif not self.project.s.PIPENV_USE_SYSTEM and not os.environ.get(
+                "VIRTUAL_ENV"
+            ):
+                os.environ["PYTHONPATH"] = self.base_paths["PYTHONPATH"]
+                os.environ.pop("PYTHONHOME", None)
             sys.path = self.sys_path
             sys.prefix = self.sys_prefix
             try:
-                yield
+                yield True
             finally:
                 sys.path = original_path
                 sys.prefix = original_prefix
-
-    @cached_property
-    def finders(self):
-        from pipenv.vendor.pythonfinder import Finder
-
-        finders = [
-            Finder(path=self.base_paths["scripts"], global_search=gs, system=False)
-            for gs in (False, True)
-        ]
-        return finders
-
-    @property
-    def finder(self):
-        return next(iter(self.finders), None)
-
-    def which(self, search, as_path=True):
-        find = operator.methodcaller("which", search)
-        result = next(iter(filter(None, (find(finder) for finder in self.finders))), None)
-        if not result:
-            result = self._which(search)
-        else:
-            if as_path:
-                result = str(result.path)
-        return result
-
-    def install(self, requirements):
-        if not isinstance(requirements, (tuple, list)):
-            requirements = [requirements]
-        with self.get_finder() as finder:
-            args = []
-            for format_control in ("no_binary", "only_binary"):
-                formats = getattr(finder.format_control, format_control)
-                args.extend(
-                    (
-                        "--" + format_control.replace("_", "-"),
-                        ",".join(sorted(formats or {":none:"})),
-                    )
-                )
-            if finder.index_urls:
-                args.extend(["-i", finder.index_urls[0]])
-                for extra_index in finder.index_urls[1:]:
-                    args.extend(["--extra-index-url", extra_index])
-            else:
-                args.append("--no-index")
-            for link in finder.find_links:
-                args.extend(["--find-links", link])
-            for _, host, _ in finder.secure_origins:
-                args.extend(["--trusted-host", host])
-            if finder.allow_all_prereleases:
-                args.append("--pre")
-            if finder.process_dependency_links:
-                args.append("--process-dependency-links")
-            args.append("--")
-            args.extend(requirements)
-            out, _ = vistir.misc.run(
-                args, return_object=False, nospin=True, block=True, combine_stderr=False
-            )
-
-    @contextlib.contextmanager
-    def uninstall(self, pkgname, *args, **kwargs):
-        """A context manager which allows uninstallation of packages from the environment
-
-        :param str pkgname: The name of a package to uninstall
-
-        >>> env = Environment("/path/to/env/root")
-        >>> with env.uninstall("pytz", auto_confirm=True, verbose=False) as uninstaller:
-                cleaned = uninstaller.paths
-        >>> if cleaned:
-                print("uninstalled packages: %s" % cleaned)
-        """
-
-        auto_confirm = kwargs.pop("auto_confirm", True)
-        verbose = kwargs.pop("verbose", False)
-        with self.activated():
-            monkey_patch = next(
-                iter(
-                    dist
-                    for dist in self.base_working_set
-                    if dist.project_name == "recursive-monkey-patch"
-                ),
-                None,
-            )
-            if monkey_patch:
-                monkey_patch.activate()
-            dist = next(
-                iter(d for d in self.get_working_set() if d.project_name == pkgname), None
-            )
-            path_set = UninstallPathSet.from_dist(dist)
-            if path_set is not None:
-                path_set.remove(auto_confirm=auto_confirm, verbose=verbose)
-            try:
-                yield path_set
-            except Exception:
-                if path_set is not None:
-                    path_set.rollback()
-            else:
-                if path_set is not None:
-                    path_set.commit()
-            if path_set is None:
-                return
